"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var app_1 = require("./app");
var render_response_1 = require("./render.response");
var redirect_response_1 = require("./redirect.response");
var skip_response_1 = require("./skip.response");
var unauthorized_response_1 = require("./unauthorized.response");
var file_response_1 = require("./file.response");
var media_entity_1 = require("./entities/media.entity");
var status_error_1 = require("./status-error");
var nodemailer_1 = require("nodemailer");
var logger_1 = require("./logger");
var mailClient;
var guard = false;
function syncronizedInit() {
    if (guard)
        return;
    guard = true;
    if (!mailClient) {
        if (!app_1.app.config.mailer.host) {
            try {
                nodemailer_1.createTestAccount(function (err, account) {
                    if (err) {
                        logger_1.logger.error(err);
                        guard = false;
                        return;
                    }
                    mailClient = nodemailer_1.createTransport({
                        host: "smtp.ethereal.email",
                        port: 587,
                        secure: false,
                        auth: {
                            user: account.user,
                            pass: account.pass // generated ethereal password
                        }
                    });
                    guard = false;
                });
            }
            catch (e) {
                guard = false;
                logger_1.logger.error(e);
            }
        }
        else {
            try {
                mailClient = nodemailer_1.createTransport(app_1.app.config.mailer);
                guard = false;
            }
            catch (e) {
                guard = false;
                logger_1.logger.error(e);
            }
        }
    }
}
var FlashType;
(function (FlashType) {
    FlashType[FlashType["primary"] = 0] = "primary";
    FlashType[FlashType["secondary"] = 1] = "secondary";
    FlashType[FlashType["success"] = 2] = "success";
    FlashType[FlashType["danger"] = 3] = "danger";
    FlashType[FlashType["warning"] = 4] = "warning";
    FlashType[FlashType["info"] = 5] = "info";
    FlashType[FlashType["light"] = 6] = "light";
    FlashType[FlashType["dark"] = 7] = "dark";
})(FlashType = exports.FlashType || (exports.FlashType = {}));
function mapFlashTypeToString(type) {
    switch (type) {
        case FlashType.primary:
            return "primary";
        case FlashType.secondary:
            return "secondary";
        case FlashType.success:
            return "success";
        case FlashType.danger:
            return "danger";
        case FlashType.warning:
            return "warning";
        case FlashType.info:
            return "info";
        case FlashType.light:
            return "light";
        case FlashType.dark:
            return "dark";
    }
}
/**
 * This class defines the basic class for any controllers. It implements a lot
 * of utility methods in order to correctly generate any response.
 */
var BaseController = /** @class */ (function () {
    function BaseController(app) {
        this.logger = logger_1.logger;
        this.app = app;
        syncronizedInit();
    }
    Object.defineProperty(BaseController.prototype, "metadata", {
        get: function () {
            return this._metadata;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * This method is called only when the constructed has been completed.
     * Since this method is async, it can be used to perform some initialization
     * that needed the use of the await keyword.  */
    BaseController.prototype.postConstructor = function () {
        return __awaiter(this, void 0, void 0, function () { return __generator(this, function (_a) {
            return [2 /*return*/];
        }); });
    };
    /**
     * Add a value to the current request context.
     * Any variable added with this method will available in the template context
     * thought the @method render method.
     * @param req the current Request
     * @param key the key of the value to add
     * @param value the value to add
     */
    BaseController.prototype.addToContext = function (req, key, value) {
        if (!req.lynxContext) {
            req.lynxContext = {};
        }
        req.lynxContext[key] = value;
    };
    /**
     * Utility method to generate an error with a status code.
     * This method should be used instead of the usual throw new Error(msg).
     * In this way, a proper HTTP status code can be used (for example, 404 or 500),
     * instead of the default 400.
     * @param status the http status code to return
     * @param message the error message
     * @return a new @type StatusError object
     */
    BaseController.prototype.error = function (status, message) {
        var err = new status_error_1.default(message);
        err.statusCode = status;
        return err;
    };
    /**
     * This method generate an url to a route starting from the route name and
     * optionally its parameters.
     * If a parameter not is used to generate the route url, it will be appended
     * as a query parameter.
     * @param name the name of the route
     * @param parameters a plain object containing the paramters for the route.
     */
    BaseController.prototype.route = function (name, parameters) {
        return this.app.route(name, parameters);
    };
    /**
     * Generate a web page starting from a template and using a generated context.
     * @param view the name of the view
     * @param req the request object
     * @param context a plain object containing any necessary data needed by the view
     */
    BaseController.prototype.render = function (view, req, context) {
        if (!view.endsWith(".njk")) {
            view = view + ".njk";
        }
        if (!context) {
            context = {};
        }
        context.req = req;
        context.flash = req.session.sessionFlash;
        for (var key in req.lynxContext) {
            context[key] = req.lynxContext[key];
        }
        delete req.session.sessionFlash;
        return new render_response_1.default(view, context);
    };
    /**
     * Redirect the current route to another
     * @param routeName the new of the target route
     * @param routeParams a plain object containing the paramters for the route.
     */
    BaseController.prototype.redirect = function (routeName, routeParams) {
        return new redirect_response_1.default(this.route(routeName, routeParams));
    };
    /**
     * Add a flash message in the current request.
     * @param msg the FlashMessage to be included
     * @param req the request
     */
    BaseController.prototype.addFlashMessage = function (msg, req) {
        var session = req.session;
        if (!session.sessionFlash) {
            session.sessionFlash = [];
        }
        session.sessionFlash.push({
            type: mapFlashTypeToString(msg.type),
            message: this.tr(msg.message, req)
        });
    };
    /**
     * Add a success flash message in the current request.
     * @param msg the string (can be localized) of the message
     * @param req the request
     */
    BaseController.prototype.addSuccessMessagge = function (msg, req) {
        this.addFlashMessage({ type: FlashType.success, message: msg }, req);
    };
    /**
     * Add an error flash message in the current request.
     * @param msg the string (can be localized) of the message
     * @param req the request
     */
    BaseController.prototype.addErrorMessage = function (msg, req) {
        this.addFlashMessage({ type: FlashType.danger, message: msg }, req);
    };
    /**
     * Generate a response suitable to file download. This method can also be
     * used to generate images of specific dimensions.
     * @param path the string path of the file, or a Media object to be downloaded
     * @param options options to correctly generate the output file
     */
    BaseController.prototype.download = function (path, options) {
        var f;
        if (path instanceof media_entity_1.default) {
            if (path.isDirectory) {
                throw new Error("unable to download a directory");
            }
            f = new file_response_1.default(path.fileName);
            f.contentType = path.mimetype;
            if (path.originalName) {
                f.fileName = path.originalName;
            }
        }
        else {
            f = new file_response_1.default(path);
        }
        if (options) {
            f.options = options;
        }
        return f;
    };
    /**
     * Generate an unauthorized response.
     */
    BaseController.prototype.unauthorized = function () {
        return new unauthorized_response_1.default();
    };
    /**
     * Generate a skip resopnse. In this particuar case, the original Express `next()`
     * will be executed, causing the controller chain to continue its execution.
     */
    BaseController.prototype.next = function () {
        return new skip_response_1.default();
    };
    /**
     * Utility method to send emails from a controller.
     * This method is similar to the `sendMail` method, but define a lower level API.
     * Indeed, it directly accepts the text and the html of the email, and not the templates urls.
     * @param dest the email destination (can also be an array of addresses)
     * @param subject the subject of the email
     * @param text the text version of the email
     * @param html the html version of the email
     */
    BaseController.prototype.sendRawMail = function (dest, subject, text, html) {
        return __awaiter(this, void 0, void 0, function () {
            var mailOptions, result, e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        mailOptions = {
                            from: this.app.config.mailer.sender,
                            to: dest,
                            subject: subject,
                            text: text,
                            html: html // html body
                        };
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, mailClient.sendMail(mailOptions)];
                    case 2:
                        result = _a.sent();
                        if (result) {
                            logger_1.logger.debug("Preview URL: %s", nodemailer_1.getTestMessageUrl(result));
                        }
                        return [2 /*return*/, true];
                    case 3:
                        e_1 = _a.sent();
                        logger_1.logger.error(e_1);
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/, false];
                }
            });
        });
    };
    /**
     * Utility method to send an email from a controller. This method is async,
     * so use the await keyword (or eventually a promise) to correctly read the
     * return value.
     * This method uses the template engine to compile the email.
     * NOTE: internally, this method uses the `sendRawMail` method.
     * @param req the current request
     * @param dest the email destination (can also be an array of addresses)
     * @param subjectTemplateString the subject of the email, that can also be a string template
     * @param textTemplate the text version of the email, referencing a path in the view folders
     * @param htmlTemplate the html version of the email, referencing a path in the view folders
     * @param context a plain object containing any necessary data needed by the view
     */
    BaseController.prototype.sendMail = function (req, dest, subjectTemplateString, textTemplate, htmlTemplate, context) {
        return __awaiter(this, void 0, void 0, function () {
            var subject, text, html;
            return __generator(this, function (_a) {
                if (!context) {
                    context = {};
                }
                context.req = req;
                subject = this.app.nunjucksEnvironment.renderString(subjectTemplateString, context);
                if (!textTemplate.endsWith(".njk")) {
                    textTemplate += ".njk";
                }
                if (!htmlTemplate.endsWith(".njk")) {
                    htmlTemplate += ".njk";
                }
                text = this.app.nunjucksEnvironment.render(textTemplate, context);
                html = this.app.nunjucksEnvironment.render(htmlTemplate, context);
                return [2 /*return*/, this.sendRawMail(dest, subject, text, html)];
            });
        });
    };
    /**
     * Utility method to obtain a translated string.
     * @param str the string key to be translated
     * @param req the original request
     */
    BaseController.prototype.tr = function (str, req) {
        return this.app.translate(str, req);
    };
    /**
     * Utility method to obtain a translated string, formatted with parameters.
     * Each parameter should be encoded as {0}, {1}, etc...
     * @param str the string key to be translated
     * @param req the original request
     * @param args the arguments to format the string
     */
    BaseController.prototype.trFormat = function (str, req) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        var translated = this.tr(str, req);
        return this.format(translated, args);
    };
    BaseController.prototype.format = function (fmt) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (!fmt.match(/^(?:(?:(?:[^{}]|(?:\{\{)|(?:\}\}))+)|(?:\{[0-9]+\}))+$/)) {
            throw new Error('invalid format string.');
        }
        return fmt.replace(/((?:[^{}]|(?:\{\{)|(?:\}\}))+)|(?:\{([0-9]+)\})/g, function (_, str, index) {
            if (str) {
                return str.replace(/(?:{{)|(?:}})/g, function (m) { return m[0]; });
            }
            else {
                if (index >= args.length) {
                    throw new Error('argument index is out of range in format');
                }
                return args[index];
            }
        });
    };
    return BaseController;
}());
exports.BaseController = BaseController;
