"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var status_error_1 = require("./status-error");
exports.BLOCK_CHAIN = "__block_chain";
/**
 * This class defines the base middleware class. Any middleware should be extends
 * this abstract class, implementing the apply method.
 */
var BaseMiddleware = /** @class */ (function () {
    function BaseMiddleware(app) {
        this.app = app;
    }
    /**
     * Utility method to generate an error with a status code.
     * This method should be used instead of the usual throw new Error(msg).
     * In this way, a proper HTTP status code can be used (for example, 404 or 500),
     * instead of the default 400.
     * @param status the http status code to return
     * @param message the error message
     * @return a new @type StatusError object
     */
    BaseMiddleware.prototype.error = function (status, message) {
        var err = new status_error_1.default(message);
        err.statusCode = status;
        return err;
    };
    return BaseMiddleware;
}());
exports.BaseMiddleware = BaseMiddleware;
