"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var colors = require("colors/safe");
var Level;
(function (Level) {
    Level[Level["error"] = 0] = "error";
    Level[Level["warn"] = 1] = "warn";
    Level[Level["info"] = 2] = "info";
    Level[Level["verbose"] = 3] = "verbose";
    Level[Level["debug"] = 4] = "debug";
    Level[Level["silly"] = 5] = "silly";
})(Level = exports.Level || (exports.Level = {}));
var Logger = /** @class */ (function () {
    function Logger() {
        this.level = Level.debug;
    }
    Object.defineProperty(Logger, "shared", {
        get: function () {
            if (!Logger.self) {
                Logger.self = new Logger();
            }
            return Logger.self;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Logger.prototype, "head", {
        get: function () {
            var now = moment();
            var d = now.format("YYYY-MM-DD HH:mm:ss");
            return "[" + d + "] ";
        },
        enumerable: true,
        configurable: true
    });
    Logger.prototype.log = function (msg) {
        var optionalParameters = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            optionalParameters[_i - 1] = arguments[_i];
        }
        this.info(msg, optionalParameters);
    };
    Logger.prototype.error = function (msg) {
        var optionalParameters = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            optionalParameters[_i - 1] = arguments[_i];
        }
        if (this.level < Level.error) {
            return;
        }
        if (optionalParameters.length > 0) {
            console.error(colors.red(this.head + msg), optionalParameters);
        }
        else {
            console.error(colors.red(this.head + msg));
        }
    };
    Logger.prototype.warn = function (msg) {
        var optionalParameters = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            optionalParameters[_i - 1] = arguments[_i];
        }
        if (this.level < Level.warn) {
            return;
        }
        if (optionalParameters.length > 0) {
            console.warn(colors.yellow(this.head + msg), optionalParameters);
        }
        else {
            console.warn(colors.yellow(this.head + msg));
        }
    };
    Logger.prototype.info = function (msg) {
        var optionalParameters = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            optionalParameters[_i - 1] = arguments[_i];
        }
        if (this.level < Level.info) {
            return;
        }
        if (optionalParameters.length > 0) {
            console.info(this.head + msg, optionalParameters);
        }
        else {
            console.info(this.head + msg);
        }
    };
    Logger.prototype.verbose = function (msg) {
        var optionalParameters = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            optionalParameters[_i - 1] = arguments[_i];
        }
        if (this.level < Level.verbose) {
            return;
        }
        if (optionalParameters.length > 0) {
            console.log(this.head + msg, optionalParameters);
        }
        else {
            console.log(this.head + msg);
        }
    };
    Logger.prototype.debug = function (msg) {
        var optionalParameters = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            optionalParameters[_i - 1] = arguments[_i];
        }
        if (this.level < Level.debug) {
            return;
        }
        if (optionalParameters.length > 0) {
            console.log(this.head + msg, optionalParameters);
        }
        else {
            console.log(this.head + msg);
        }
    };
    Logger.prototype.silly = function (msg) {
        var optionalParameters = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            optionalParameters[_i - 1] = arguments[_i];
        }
        if (this.level < Level.silly) {
            return;
        }
        if (optionalParameters.length > 0) {
            console.log(this.head + msg, optionalParameters);
        }
        else {
            console.log(this.head + msg);
        }
    };
    return Logger;
}());
exports.default = Logger;
var logger = Logger.shared;
exports.logger = logger;
